@@ -67,8 +67,6 @@ public Q_SLOTS:
     void cursorUp();
 
 public:
-    void tab(bool shift);
-
     void toggleDocumentation();
 
     const KateCompletionModel *model() const;
@@ -82,6 +80,10 @@ public:
     int automaticInvocationDelay() const;
     void setAutomaticInvocationDelay(int delay);
 
+    void setIgnoreBufferSignals(bool ignore);
+
+    bool m_ignoreBufferSignals = false;
+
     struct CompletionRange {
         CompletionRange()
         {
@@ -112,8 +114,7 @@ public:
 
     QWidget *currentEmbeddedWidget();
 
-    // Returns true if a screen border has been hit
-    bool updatePosition(bool force = false);
+    void updatePosition(bool force = false);
 
     bool eventFilter(QObject *watched, QEvent *event) override;
 
