@@ -6,23 +6,11 @@ var katescript = {
     "kate-version": "5.1"
 }; // kate-script-header, must be at the start of the file without comments, pure json
 
-/**
- * This file is part of the Kate Project.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License version 2 as published by the Free Software Foundation.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- */
+/*
+    This file is part of the Kate Project.
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 // required katepart js libraries
 require ("range.js");
@@ -53,6 +41,7 @@ function dbg() {
     }
 }
 
+const multiLineStringLiteralStartRegex = /R"(.+)?\(/
 
 //BEGIN global variables and functions
 // maximum number of lines we look backwards/forward to find out the indentation
@@ -212,48 +201,46 @@ function lastNonSpace(text)
     return -1;
 }
 
-// /**
-//  * Check, whether the beginning of line is inside a "..." string context.
-//  * Note: the function does not check for comments
-//  * return: leading whitespaces as string, or null if not in a string
-//  */
-// function inString(line)
-// {
-//     var currentLine = line;
-//     var currentString;
-//
-//     // go line up as long as the previous line ends with an escape character '\'
-//     while (currentLine >= 0) {
-//         currentString = document.line(currentLine - 1);
-//         if (currentString.charAt(document.lastColumn(currentLine - 1)) != '\\')
-//             break;
-//         --currentLine;
-//     }
-//
-//     // iterate through all lines and toggle bool insideString for every quote
-//     var insideString = false;
-//     var indentation = null;
-//     while (currentLine < line) {
-//         currentString = document.line(currentLine);
-//         var char1;
-//         var i;
-//         var lineLength = document.lineLength(currentLine);
-//         for (i = 0; i < lineLength; ++i) {
-//             char1 = currentString.charAt(i);
-//             if (char1 == "\\") {
-//                 // skip escaped character
-//                 ++i;
-//             } else if (char1 == "\"") {
-//                 insideString = !insideString;
-//                 if (insideString)
-//                     indentation = currentString.substring(0, document.firstColumn(currentLine) + 1);
-//             }
-//         }
-//         ++currentLine;
-//     }
-//
-//     return insideString ? indentation : null;
-// }
+/**
+ * Don't indent when in a multiline string literal of form "something\
+nextline"
+ * Note: the function does not check for comments
+ */
+function tryString(line)
+{
+    var currentLine = line;
+    var currentString;
+
+    // go line up as long as the previous line ends with an escape character '\'
+    while (currentLine >= 0) {
+        currentString = document.line(currentLine - 1);
+        if (currentString.charAt(document.lastColumn(currentLine - 1)) != '\\')
+            break;
+        --currentLine;
+    }
+
+    // iterate through all lines and toggle bool insideString for every quote
+    var insideString = false;
+    var indentation = -1;
+    while (currentLine < line) {
+        currentString = document.line(currentLine);
+        var char1;
+        var i;
+        var lineLength = document.lineLength(currentLine);
+        for (i = 0; i < lineLength; ++i) {
+            char1 = currentString.charAt(i);
+            if (char1 == "\\") {
+                // skip escaped character
+                ++i;
+            } else if (char1 == "\"") {
+                insideString = !insideString;
+            }
+        }
+        ++currentLine;
+    }
+
+    return insideString ? 0 : -1;
+}
 
 /**
  * C comment checking. If the previous line begins with a "/*" or a "* ", then
@@ -433,7 +420,7 @@ function tryCKeywords(line, isBrace)
 
     // found non-empty line
     var currentString = document.line(currentLine);
-    if (currentString.search(/^\s*(if\b|for|do\b|while|switch|[}]?\s*else|((private|public|protected|case|default|signals|Q_SIGNALS).*:))/) == -1)
+    if (currentString.search(/^\s*(if\b|for(each)?|do\b|while|switch|[}]?\s*else(if)?|((private|public|protected|case|default|signals|Q_SIGNALS).*:))/) == -1)
         return -1;
     dbg("Found first word: " + RegExp.$1);
     lastPos = document.lastColumn(currentLine);
@@ -460,7 +447,18 @@ function tryCKeywords(line, isBrace)
         //     --b)
         cursor = document.anchor(currentLine, lastPos, '(');
         if (cursor.isValid()) {
-            indentation = document.toVirtualColumn(cursor.line, cursor.column + 1);
+            // Same line, we know there is a keyword here
+            if (cursor.line == line) {
+                indentation = document.toVirtualColumn(cursor.line, cursor.column + 1);
+            } else {
+                // check that the returned cursor's line contains
+                // a keyword and isn't a func call.
+                var cursorLine = document.line(cursor.line);
+                var s = cursorLine.search(/^\s*(if\b|for(each)?|do\b|while|switch|[}]?\s*else(if)?)/);
+                if (s != -1) {
+                    indentation = document.toVirtualColumn(cursor.line, cursor.column + 1);
+                }
+            }
         }
     }
 
@@ -468,6 +466,27 @@ function tryCKeywords(line, isBrace)
     return indentation;
 }
 
+/**
+ * Is this line a single statement of a condition:
+ * if (true)
+ *     doWork();
+ * - Finds the previous non empty line
+ * - Checks if it has control keyword
+ * - Checks if it doesn't have brace
+ */
+function isSingleStmtCondition(line)
+{
+    if (line > 1) {
+        var prev = lastNonEmptyLine(line - 1);
+        var prevText = document.line(prev);
+        var hasKeyword = prevText.search(/^\s*(if\b|[}]?\s*else(if)?\b|do\b|while\b|for(each)?\b)/);
+        if (hasKeyword != -1 && !prevText.contains("{")) {
+            return true;
+        }
+    }
+    return false;
+}
+
 /**
  * Search for if, do, while, for, ... as we want to indent then.
  * Return null, if nothing useful found.
@@ -486,7 +505,7 @@ function tryCondition(line)
     var indentation = -1;
 
     if (lastChar == ';'
-        && currentString.search(/^\s*(if\b|[}]?\s*else|do\b|while\b|for)/) == -1)
+        && currentString.search(/^\s*(if\b|[}]?\s*else(if)?\b|do\b|while\b|for(each)?\b)/) == -1)
     {
         // idea: we had something like:
         //   if/while/for (expression)
@@ -497,6 +516,18 @@ function tryCondition(line)
         if (currentIndentation == 0)
             return -1;
 
+        // Is this a condition with only one stmt in it
+        var singleStmt = isSingleStmtCondition(currentLine);
+        if (singleStmt) {
+            var leftBrace = findLeftBrace(currentLine, 0);
+            if (leftBrace != -1) {
+                indentation = leftBrace + gIndentWidth;
+                dbg("tryCondition(1): success in line " + currentLine);
+                return indentation;
+            }
+        }
+
+
         var lineDelimiter = 10; // 10 limit search, hope this is a sane value
         while (currentLine > 0 && lineDelimiter > 0) {
             --currentLine;
@@ -507,7 +538,7 @@ function tryCondition(line)
 
             if (firstPosVirtual < currentIndentation) {
                 currentString = document.line(currentLine);
-                if (currentString.search(/^\s*(if\b|[}]?\s*else|do\b|while\b|for)[^{]*$/) != -1)
+                if (currentString.search(/^\s*(if\b|[}]?\s*else(if)?|do\b|while\b|for(each)?)[^{]*$/) != -1)
                     indentation = firstPosVirtual;
                 break;
             } else if (currentLine == 0 || lineDelimiter == 0) {
@@ -516,7 +547,7 @@ function tryCondition(line)
         }
     }
 
-    if (indentation != -1) dbg("tryCondition: success in line " + currentLine);
+    if (indentation != -1) dbg("tryCondition(2): success in line " + currentLine);
     return indentation;
 }
 
@@ -677,7 +708,7 @@ function tryMatchedAnchor(line, alignOnly)
         var actualIndentation = document.firstVirtualColumn(line - 1);
         var indentation = -1;
         if ( expectedIndentation <= actualIndentation ) {
-            if ( lastChar == ',' ) {
+            if ( lastChar == ',' && char != ')' ) {
                 // use indentation of last line instead and place closing anchor
                 // in same column of the openeing anchor
                 document.insertText(line, document.firstColumn(line), "\n");
@@ -695,12 +726,16 @@ function tryMatchedAnchor(line, alignOnly)
                 indentation = document.firstVirtualColumn(closingAnchor.line);
             } else {
                 // otherwise don't add a new line, just align on closing anchor
-                indentation = document.toVirtualColumn(closingAnchor);
+                indentation = document.toVirtualColumn(closingAnchor) + 1;
             }
             dbg("tryMatchedAnchor: success in line " + closingAnchor.line);
             return indentation;
+        } else if ( lastChar == ',') {
+            dbg("tryMatchedAnchor: success in line " + closingAnchor.line);
+            return document.toVirtualColumn(closingAnchor) + 1;
         }
     }
+
     // otherwise we i.e. pressed enter between (), [] or when we enter before curly brace
     // increase indentation and place closing anchor on the next line
     indentation = document.firstVirtualColumn(closingAnchor.line);
@@ -712,6 +747,57 @@ function tryMatchedAnchor(line, alignOnly)
     return indentation + gIndentWidth;
 }
 
+function escapeForRegex(string) {
+    return string.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
+}
+
+/**
+ * Don't indent on multiline string literals of form R"[optional delimiter text]()[optional delimiter text]"
+ */
+function tryMultiLineStringLiteral(line) {
+    var currentLine = line;
+    var currentString;
+    var delim = "";
+    var found = false;
+
+    // go line up as long as the previous line ends with an escape character '\'
+    // go line up until we find something of form R"[delim](
+    while (currentLine >= 0) {
+        currentString = document.line(currentLine - 1);
+        var startMatch = currentString.match(multiLineStringLiteralStartRegex);
+        if (startMatch !== null) {
+            found = true;
+            delim = startMatch[1];
+            break;
+        }
+        --currentLine;
+    }
+
+    if (!found) {
+        return -1;
+    }
+
+    // iterate through all lines and toggle bool insideString for every quote
+    var insideString = true;
+    var endReached = false;
+    var indentation = null;
+    const multiLineStringLiteralEndRegex = new RegExp('\\)' + escapeForRegex(delim) + '"');
+    while (currentLine < line) {
+        currentString = document.line(currentLine);
+        dbg(currentString);
+        var endMatch = currentString.match(multiLineStringLiteralEndRegex);
+        if (endMatch !== null) {
+            dbg("End reached: " + currentString);
+            endReached = true;
+            break;
+        }
+        ++currentLine;
+    }
+    indentation = document.firstVirtualColumn(line);
+    dbg(endReached ? -1 : indentation);
+    return endReached ? -1 : indentation;
+}
+
 /**
  * Indent line.
  * Return filler or null.
@@ -729,6 +815,10 @@ function indentLine(line, alignOnly)
         filler = tryCComment(line);
     if (filler == -1 && !alignOnly)
         filler = tryCppComment(line);
+    if (filler == -1)
+        filler = tryMultiLineStringLiteral(line);
+    if (filler == -1)
+        filler = tryString(line);
     if (filler == -1)
         filler = trySwitchStatement(line);
     if (filler == -1)
@@ -785,7 +875,7 @@ function processChar(line, c)
     } else if (firstPos == column - 1 && c == '}'  && firstPos > prevFirstPos) {
         // align indentation to previous line when creating new block with auto brackets enabled
         // prevents over-indentation for if blocks and loops
-        return prevFirstPos;
+        return document.toVirtualColumn(line - 1, prevFirstPos);
     } else if (cfgSnapSlash && c == '/' && lastPos == column - 1) {
         // try to snap the string "* /" to "*/"
         var currentString = document.line(line);
